/*
	This linker script links the object files when generating the ELF output.
	`romdata`, `[be]bss`, `[be]data` symbols are used in the runtime
	initialization routine (crt0.c) to initialize the application.

	Two RAM areas are specified. The "real ram" is accessed directly while the
	other area is virtualized. It is not possible to execute code in
	virtualized ram -- for an add-in, I don't know if this is also the case
	for the UpdateExe, so I'll just take one part of the RAM.

	An end block is located at the end of the binary - it's in the `end`
	section.
*/

OUTPUT_ARCH(sh3)
ENTRY(_start)

MEMORY
{
	/* http://bible.planet-casio.com/simlo/chm/v20/CommunicationsProtocol.htm */
	rom : o = 0x88030000, l = 64k
	ram : o = 0x08024000, l = 40k
	realram : o = 0x0802e000, l = 8k
}

SECTIONS
{
	/* ROM sections : binary code and read-only data */
	.text : {
		*(.pretext.entry)
		*(.pretext)

		_bctors = . ;
		*(.ctors)
		_ectors = . ;
		_bdtors = . ;
		*(.dtors)
		_edtors = . ;

		*(.text)
		*(.text.*)
	} > rom

	.rodata : {
		*(.rodata.fxconv)
		*(.rodata)
		*(.rodata.*)

		_romdata = ALIGN(4) ;
	} > rom

	/* RAM sections : bss and rw data */
	.bss : {
		_bbss = . ;
		*(.bss)
		_ebss = . ;
	} > ram

	.data : AT(_romdata) ALIGN(4) {
		_bdata = . ;
		*(.data)
		*(.data.*)
		_edata = . ;
	} > ram

	.cc : AT(_romdata + SIZEOF(.data)) ALIGN(4) {
		*(.eh_frame)
		*(.jcr)

		_gint_data = _romdata + SIZEOF(.data) + SIZEOF(.cc) ;
	} > ram

	/* Real RAM: interrupt, exception and TLB miss handlers */
	.gint : AT(_gint_data) {
		/* The vbr needs to be 0x100-aligned because of an ld issue. */
		. = ALIGN(0x100) ;
		_gint_vbr = . ;
		_bgint = . ;

		/* Exception handler. */
		. = _gint_vbr + 0x100 ;
		*(.gint.exc.entry)
		*(.gint.exc)

		/* TLB miss handler. */
		. = _gint_vbr + 0x400 ;
		*(.gint.tlb.entry)
		*(.gint.tlb)

		/* Interrupt handler. */
		. = _gint_vbr + 0x600 ;
		*(.gint.int.entry)
		*(.gint.int)

		_egint = . ;
	} > realram

	.end : AT(_gint_data + SIZEOF(.gint)) {
		*(.end)
	} > rom
}
